//
//  P4DepotDir.swift
//  DocHub
//
//  Created by Tristan Juricek on 6/25/14.
//  Copyright (c) 2014 Perforce. All rights reserved.
//

import Foundation

class P4DepotDir : NSObject, Directory {
    
    var p4DepotArea:P4DepotArea
    
    var area:Area {
        return p4DepotArea
    }

    var p4Dir:P4Dir
    
    var name:String {
        return p4Dir.name
    }
    
    var parent:Directory? = nil
    
    var children:Directory[]
    
    func loadChildren() -> Directory[] {
        // TODO this should be lazy loaded once
        var depotDirs = Array<P4DepotDir>()
        let depotPath = "\(p4Dir.depotPath)/*"
        p4DepotArea.p4Library.withConnection({
            (api:P4ClientApi) -> () in
            
            let (err, dirs) = api.dirs(depotPath)
            
            if let e = err {
                NSException(name: "DirectoryListFailed",
                    reason: e.localizedDescription, userInfo: nil).raise()
            }
            
            for p4dir in dirs {
                let dir = P4DepotDir(depotArea:self.p4DepotArea, dir:p4dir)
                dir.parent = self
                depotDirs.append(dir)
            }
            })
        return depotDirs
    }
    
    var files:File[]
    
    func loadFiles() -> File[] {
        // TODO this should be lazy loaded once
        var depotFiles = Array<P4DepotFile>()
        let depotPath = "\(p4Dir.depotPath)/*"
        
        p4DepotArea.p4Library.withConnection({
            (api:P4ClientApi) -> () in
            
            let (err, p4Files) = api.files(depotPath)
            
            if let e = err {
                NSException(name: "FileListFailed",
                    reason: e.localizedDescription, userInfo: nil).raise()
            }
            
            for p4file in p4Files {
                let file = P4DepotFile(depotArea:self.p4DepotArea, p4File:p4file)
                file.parent = self
                depotFiles.append(file)
            }
        })
        
        return depotFiles
    }
    
    var sorted:Namable[] = []
    
    init(depotArea:P4DepotArea, dir:P4Dir) {
        self.p4DepotArea = depotArea
        self.p4Dir = dir
        
        children = []
        files = []
        super.init()
        // TODO the @lazy attributes would be ideal, but I think I hit a 
        //      compiler bug in the beta release
        children = loadChildren()
        files = loadFiles()
        sorted = sortItems()
    }
    
    func sortItems() -> Namable[] {
        var namables = Array<Namable>()
        for child in children {
            namables.append(child)
        }
        for file in files {
            namables.append(file)
        }
        let newArr : Array<Namable> = sort(namables, {
            (n1, n2) -> Bool in
            return n1.name.compare(n2.name) < 0
        })
        return newArr
    }
    
    func itemAt(index:Int) -> Namable {
        return sorted[index]
    }
}